<?php
// +----------------------------------------------------------------------+
// | Anuko Time Tracker
// +----------------------------------------------------------------------+
// | Copyright (c) Anuko International Ltd. (http://www.anuko.com)
// +----------------------------------------------------------------------+
// | LIBERAL FREEWARE LICENSE: This source code document may be used
// | by anyone for any purpose, and freely redistributed alone or in
// | combination with other software, provided that the license is obeyed.
// |
// | There are only two ways to violate the license:
// |
// | 1. To redistribute this code in source form, with the copyright
// |    notice or license removed or altered. (Distributing in compiled
// |    forms without embedded copyright notices is permitted).
// |
// | 2. To redistribute modified versions of this code in *any* form
// |    that bears insufficient indications that the modifications are
// |    not the work of the original author(s).
// |
// | This license applies to this document only, not any other software
// | that it may be combined with.
// |
// +----------------------------------------------------------------------+
// | Contributors:
// | http://www.anuko.com/content/time_tracker/open_source/credits.htm
// +----------------------------------------------------------------------+

// Class ttUserHelper is a future replacement for UserHelper class.
// We are trying to do some code refactoring to optimize some things.
// The objectives of refactoring are:
// - Make coe execution faster.
// - Make code more manageable.
// - Make code easier to understand.
class ttUserHelper {

  // The delete function deletes a user. Current authenticated user is assumed to be this user manager.
  static function delete($user_id) {
  	
  	global $userMin;
  	
  	// Preliminary checks. Only managers and co-managers can delete users.
  	if (!$userMin->isManager && !$userMin->isComanager) {
      return false;
  	}
  	
  	$user_count = ttUserHelper::getUserCount($userMin->manager_id);
   	// Deleting a manager with active users is not allowed.
   	$is_manager = ($user_id == $userMin->manager_id);
    if ($is_manager && ($user_count > 0))
      return false;

    $mdb2 = getConnection();
  	
  	$deleted = DELETED;
    if (!$user_count) {
      // No users.
      if ($is_manager) {
      	//  Mark team projects as deleted.
        $sql = "update projects set p_status = $deleted where p_manager_id = $user_id";
        $affected = &$mdb2->exec($sql);
        if (PEAR::isError($affected) != 0) {
          return false;
        }
 
        // Mark team activities as deleted.
        $sql = "update activities set a_status = $deleted where a_manager_id = $user_id";
        $affected = &$mdb2->exec($sql);
        if (PEAR::isError($affected) != 0) {
          return false;
        }
      }
    }
    
    // Delete user binds (to projects).
    $sql = "delete from user_bind where ub_id_u = $user_id";
    $affected = &$mdb2->exec($sql);
    if (PEAR::isError($affected) != 0) {
      return false;
    }
 
    // Delete user.
    $sql = "update users set u_active = $deleted where u_id = $user_id
      and (u_manager_id = ".$userMin->manager_id." or u_id = ".$userMin->manager_id.")";
    $affected = &$mdb2->exec($sql);
    if (PEAR::isError($affected) != 0) {
      return false;
    } else {
      return true;
    }
  }
  
  // The getUserCount function returns number of people in team excluding manager.
  static function getUserCount($manager_id) {
  	$mdb2 = getConnection();
    
    $sql = "select count(u_id) as cnt from users where u_manager_id = $manager_id and u_active = 1";
    $res = &$mdb2->query($sql);

    if (PEAR::isError($res) == 0) {
      $val = $res->fetchRow();
      return $val['cnt'];
    }
    return false;
  }
}
?>