<?php
// +----------------------------------------------------------------------+
// | Anuko Time Tracker
// +----------------------------------------------------------------------+
// | Copyright (c) Anuko International Ltd. (http://www.anuko.com)
// +----------------------------------------------------------------------+
// | LIBERAL FREEWARE LICENSE: This source code document may be used
// | by anyone for any purpose, and freely redistributed alone or in
// | combination with other software, provided that the license is obeyed.
// |
// | There are only two ways to violate the license:
// |
// | 1. To redistribute this code in source form, with the copyright
// |    notice or license removed or altered. (Distributing in compiled
// |    forms without embedded copyright notices is permitted).
// |
// | 2. To redistribute modified versions of this code in *any* form
// |    that bears insufficient indications that the modifications are
// |    not the work of the original author(s).
// |
// | This license applies to this document only, not any other software
// | that it may be combined with.
// |
// +----------------------------------------------------------------------+
// | Contributors:
// | http://www.anuko.com/content/time_tracker/open_source/credits.htm
// +----------------------------------------------------------------------+

// ttUser class is a planned replacement of the User class.
// This is a refactoring attempt to re-organize and optimize variables we store in $_SESSION.
class ttUser {
  var $login = null;        // User login.
  var $name = null;         // User name.
  var $id = null;           // User id.
  var $company_id = null;   // Company id.
  var $behalf_id = null;    // User, on behalf of whom we are working.
  var $behalf_name = null;  // User name, on behalf of whom we are working.
  var $manager_id = null;   // Manager id.
  var $isManager = false;   // True if user is manager.
  var $isComanager = false; // True if user is co-manager.
  var $isAdmin = false;     // True if user is administrator.
  var $show_pie = null;     // To show pie chart or not.
  var $pie_mode = null;     // Mode of the pie chart.
  var $lang = null;         // User language.
  var $details = null;		// User details string to display.
  
  // Constructor.
  function ttUser($login, $id = null) {
  	if (!$login && !$id) {
      // nothing to initialize
  	  return;
  	}
  	
    $mdb2 = getConnection();
  	
  	$sql = "SELECT u_id, u_login, u_name, u_company_id, u_manager_id, u_comanager, u_show_pie, u_pie_mode, u_lang FROM users WHERE ";
  	if ($id)
  	  $sql .= "u_id = $id";
    else
  	  $sql .= "u_login = ".$mdb2->quote($login);
  	$sql .= " AND u_active = 1";
  	
    $res = &$mdb2->query($sql);
    if (PEAR::isError($res)) {
      return;
      //die($res->getMessage());
    }

    $val = $res->fetchRow();
    if ($val['u_id'] > 0) {
      $this->login = $val['u_login'];
      $this->name = $val['u_name'];
      $this->id = $val['u_id'];
      $this->company_id = $val['u_company_id'];
      $this->show_pie = $val['u_show_pie'];
      $this->pie_mode = $val['u_pie_mode'];
      $this->lang = $val['u_lang'];
      if ($val['u_comanager']) {
      	$this->isComanager = true;
      }
      if ($val['u_manager_id'] > 0) {
        $this->manager_id = $val['u_manager_id'];
      } else {
      	$this->manager_id = $this->id;
      	$this->isManager = true;
        if ($login == "admin") {
      	  $this->isAdmin = true;
        }
      }
      
      // Set "on behalf" id and name. TODO: this needs to be refactored when we re-organize session variables
      // and get rid of the entire _usersession block.
      if (isset($_SESSION['_usersession']["behalfid"])) {
          $this->behalf_id = $_SESSION['_usersession']["behalfid"];
          $this->behalf_name = @$_SESSION['_usersession']["behalfname"];
      }
    }
  }
  
  // The setUserDetails() function initializes user details string.
  // It needs $i18n class lo be loaded.
  function setUserDetails() {
  	global $i18n;
    $manager_info = "";
    if ($this->isAdmin) {
      $manager_info = $i18n->getKey('label.painfo');
    } else if ($this->isManager) {
      $manager_info = $i18n->getKey('label.pminfo');
    } else if ($this->isComanager) {
      $manager_info = $i18n->getKey('label.pcminfo');
    }

    if($this->behalf_id > 0) {
      $user_str = sprintf($i18n->getKey('label.pbehalf_info'), $this->name, $manager_info, $this->behalf_name);
    } else {
      $user_str = sprintf("%s %s", $this->name, $manager_info);
    }
    $user_str = trim($user_str);
    
    // Get company details.
    if ($this->company_id > 0) {	
      $mdb2 = getConnection();
  	  $sql = "select c_name, c_www from companies where c_id = $this->company_id";
  	  $res = &$mdb2->query($sql);
      if (PEAR::isError($res)) {
        die($res->getMessage());
      }

      $val = $res->fetchRow();
      $company_name = trim($val['c_name']);
      $company_www = trim($val['c_www']);
      
      $www = $company_www;
      if (substr($www, 0, 7) == "http://") {
        $www = substr($www, 7);
      }
      if (substr($www, -1) == "/") {
        $www = substr($www, 0, strlen($www) -1);
      }
      $company_str = $company_name;
      if ($www) {
      	$company_str .= " (<a href='".$company_www."' target=\"_blank\">".$www."</a>)";
      }
    }
    
    // Set the user details string.
    if ($company_str) {
      $this->details = sprintf($i18n->getKey('label.pinfo'), $user_str, $company_str);
    } else {
      $this->details = sprintf($i18n->getKey('label.pinfo2'), $user_str);
    }
  }
}
?>