<?php
// +----------------------------------------------------------------------+
// | Anuko Time Tracker
// +----------------------------------------------------------------------+
// | Copyright (c) Anuko International Ltd. (http://www.anuko.com)
// +----------------------------------------------------------------------+
// | LIBERAL FREEWARE LICENSE: This source code document may be used
// | by anyone for any purpose, and freely redistributed alone or in
// | combination with other software, provided that the license is obeyed.
// |
// | There are only two ways to violate the license:
// |
// | 1. To redistribute this code in source form, with the copyright
// |    notice or license removed or altered. (Distributing in compiled
// |    forms without embedded copyright notices is permitted).
// |
// | 2. To redistribute modified versions of this code in *any* form
// |    that bears insufficient indications that the modifications are
// |    not the work of the original author(s).
// |
// | This license applies to this document only, not any other software
// | that it may be combined with.
// |
// +----------------------------------------------------------------------+
// | Contributors:
// | http://www.anuko.com/content/time_tracker/open_source/credits.htm
// +----------------------------------------------------------------------+

class Validator {

    var $errorMsg;
    var $string;

    function Validator ($value = "") {
    	$this->string = $value;
        $this->errorMsg = array();
        $this->validate();
    }

    function validate() {
        // Superclass method does nothing
    }

    function setError($msg) {
        $this->errorMsg[]=$msg;
    }

    function isValid() {
        if ( count($this->errorMsg)>0 ) {
            return false;
        } else {
            return true;
        }
    }

    function getError() {
        return array_pop($this->errorMsg);
    }
    
    function validateEmptyString() {
    	if (strlen($this->string) == 0  ) {
            $this->setError('Value is empty');
        }
    }
    
    // Validates for a trimmed string to have some characters.
    function validateString() {
    	if (strlen(trim($this->string)) == 0  ) {
            $this->setError('Value is empty');
        }
    }
    
	// Validates an integer value.
	function validateInteger() {
	    if (!preg_match('/^[0-9]+$/', $this->string )) {
            $this->setError('Value contains invalid characters');
        }
	}
	
	// Validates a float value.
	function validateFloat($decimal = "") {
		if (!$decimal) $decimal = ".";
		$this->validateEmptyString();
	    if (!preg_match('/^[0-9'.$decimal.']+$/', $this->string )) {
            $this->setError('Value contains invalid characters');
        }
	}
	
	// Validates a single email address.
	function validateEmail() {
		// This should accept a string with e-mail addresses, possible separated by a comma with spaces...
		if (!preg_match('/^[_a-zA-Z\d\-\.]+@([_a-zA-Z\d\-]+(\.[_a-zA-Z\d\-]+)+)$/', $this->string )) {
          $this->setError('Value contains invalid characters');
        }
	}

	// Validates a list of email addresses separated by a comma with optional spaces.
	function validateEmailList() {
		if (!preg_match('/^[_a-zA-Z\d\-\.]+@([_a-zA-Z\d\-]+(\.[_a-zA-Z\d\-]+)+)(,\s*[_a-zA-Z\d\-\.]+@([_a-zA-Z\d\-]+(\.[_a-zA-Z\d\-]+)+))*$/', $this->string )) {
            $this->setError('Value contains invalid characters');
        }
	}

	// Validates a date string.
	function validateDate() {
		// This should accept a string in format 'YYYY-MM-DD', 'MM/DD/YYYY', 'DD.MM.YYYY', or 'DD.MM.YYYY whatever'.
		if ((!preg_match('/^\d\d\d\d-\d\d-\d\d$/', $this->string ))
          && (!preg_match('/^\d\d\/\d\d\/\d\d\d\d$/', $this->string ))
          && (!preg_match('/^\d\d\.\d\d\.\d\d\d\d$/', $this->string ))
          && (!preg_match('/^\d\d\.\d\d\.\d\d\d\d .+$/', $this->string ))) {
          $this->setError('Value contains invalid characters');
        }
	}
}
?>