<?php
// +----------------------------------------------------------------------+
// | Anuko Time Tracker
// +----------------------------------------------------------------------+
// | Copyright (c) Anuko International Ltd. (http://www.anuko.com)
// +----------------------------------------------------------------------+
// | LIBERAL FREEWARE LICENSE: This source code document may be used
// | by anyone for any purpose, and freely redistributed alone or in
// | combination with other software, provided that the license is obeyed.
// |
// | There are only two ways to violate the license:
// |
// | 1. To redistribute this code in source form, with the copyright
// |    notice or license removed or altered. (Distributing in compiled
// |    forms without embedded copyright notices is permitted).
// |
// | 2. To redistribute modified versions of this code in *any* form
// |    that bears insufficient indications that the modifications are
// |    not the work of the original author(s).
// |
// | This license applies to this document only, not any other software
// | that it may be combined with.
// |
// +----------------------------------------------------------------------+
// | Contributors:
// | http://www.anuko.com/content/time_tracker/open_source/credits.htm
// +----------------------------------------------------------------------+

  require_once('initialize.php');
  import('form.Form');
  import('UserHelper');
  import('TimeHelper');
  import('DateAndTime');
  import('SysConfig');
  
  // Authentication check and redirect.
  if ($auth->isAuthenticated()) {
    $user = new User($auth->getUserId());
    if ($user->isAdministrator()) {
      Header("Location: admin.php");
      exit();
    }
  } else {
    Header("Location: login.php");
    exit();
  }
    
  $cl_ts = $request->getParameter('ts', (isset($_SESSION['ts'])? $_SESSION['ts'] : null));
  $_SESSION['ts'] = $cl_ts;

  $time_rec = TimeHelper::findTimeRecord($user->getActiveUser(), $cl_ts);
        
  if ($request->getMethod()=="POST") {
    if ($request->getParameter('confirmation'))  {  // Confirmation of deleting
  
      // Determine if it's okay to delete the record.
      $rec_date = $time_rec['al_date'];

      $locktime = $user->getLocktime();
      $lockdate = 0;
      if ($locktime < 0 || $locktime == null || $locktime == "") {
        $sc = new SysConfig(new User($user->getOwnerId(), false));
	    $locktime = $sc->getValue(SYSC_LOCK_DAYS);
      }
      if ($locktime > 0) {
        $lockdate = new DateAndTime($i18n->getDateTimeFormat(), time());
        $lockdate->decDay($locktime);
      }
      $cr_date = new DateAndTime(DB_DATEFORMAT);
      $cr_date->parseVal($rec_date, DB_DATEFORMAT);
      
      // Determine if the record is uncompleted.
      $uncompleted = ($time_rec['tdur'] == '0:00');
      	
      if($lockdate && $cr_date->before($lockdate) && !$uncompleted) {
        $errors->add("lock_period", $i18n->getKey("errors.period_lock"));
      }
           
      if ($errors->isEmpty()) {
        
        // Delete the record.
        if (TimeHelper::delete($user->getActiveUser(), $cl_ts)) {
          header("Location: mytime.php");
          exit();
        } else {
          $errors->add("deleting",$i18n->getKey("errors.mt_del_no"));
        }
      }
    }
    if ($request->getParameter('rejecting')) {
      header("Location: mytime.php");
      exit();
    }
  }
		
  $form = new Form('mytimeForm');
  $form->addInput(array("type" => "hidden", "name" => "ts", "value" => $cl_ts));
  $form->addInput(array("type" => "submit", "name" => "confirmation", "value" => $i18n->getKey('button.delete')));
  $form->addInput(array("type" => "submit", "name" => "rejecting", "value" => $i18n->getKey('button.cancel')));

  $smarty->assign("time_rec", array($time_rec));
  $smarty->assignByRef("errors", $errors);
  $smarty->assignByRef("messages", $messages);
  $smarty->assign("forms", array($form->getName() => $form->toArray()));
  $smarty->assign("userdet_string", UserHelper::getUserDetailsString($user, $GLOBALS["I18N"]));
  $smarty->assign("title_page", $i18n->getKey("form.mytime.del_str"));
  $smarty->assign("content_page_name", "mytime_del.tpl");
  $smarty->display(INDEX_TEMPLATE);
?>