<?php
// +----------------------------------------------------------------------+
// | Anuko Time Tracker
// +----------------------------------------------------------------------+
// | Copyright (c) Anuko International Ltd. (http://www.anuko.com)
// +----------------------------------------------------------------------+
// | LIBERAL FREEWARE LICENSE: This source code document may be used
// | by anyone for any purpose, and freely redistributed alone or in
// | combination with other software, provided that the license is obeyed.
// |
// | There are only two ways to violate the license:
// |
// | 1. To redistribute this code in source form, with the copyright
// |    notice or license removed or altered. (Distributing in compiled
// |    forms without embedded copyright notices is permitted).
// |
// | 2. To redistribute modified versions of this code in *any* form
// |    that bears insufficient indications that the modifications are
// |    not the work of the original author(s).
// |
// | This license applies to this document only, not any other software
// | that it may be combined with.
// |
// +----------------------------------------------------------------------+
// | Contributors:
// | http://www.anuko.com/content/time_tracker/open_source/credits.htm
// +----------------------------------------------------------------------+

  require_once('initialize.php');
  import('form.Form');
  import('UserHelper');
  import('TimeHelper');
  import('DateAndTime');
  import('SysConfig');
  import('form.check.Validator');

  // Authentication check and redirect.
  if ($auth->isAuthenticated()) {
    $user = new User($auth->getUserId());
    if ($user->isAdministrator()) {
      Header("Location: admin.php");
      exit();
    }
  } else {
    Header("Location: login.php");
    exit();
  }
  
  $cl_ts = $request->getParameter('ts', (isset($_SESSION['ts'])? $_SESSION['ts'] : null));
  $_SESSION['ts'] = $cl_ts;

  // Get the time record we are editing.
  $time_rec = TimeHelper::findTimeRecord($user->getActiveUser(), $cl_ts);
  
  // Determine record creation date.
  $rec_date = $time_rec['al_date'];
  $cr_date = new DateAndTime(DB_DATEFORMAT);
  $cr_date->parseVal($rec_date, DB_DATEFORMAT);

  // Initialize variables.
  $cl_start = $cl_finish = $cl_duration = $cl_pr_date = $cl_note = $cl_project = $cl_activity = $cl_billable = null;
  if ($request->getMethod() == "POST") {
    $cl_start = trim($request->getParameter('start'));
    $cl_finish = trim($request->getParameter('finish'));
    $cl_duration = trim($request->getParameter('duration'));
    $cl_pr_date	= $request->getParameter('pr_date');
    $cl_note = trim($request->getParameter('note'));
    $cl_project = $request->getParameter('project');
    $cl_activity = $request->getParameter('activity');
    $cl_billable = $request->getParameter('billable');
  } else {
    $cl_project = $time_rec["al_project_id"];
    $cl_activity = $time_rec["al_activity_id"];
    $cl_start = $time_rec["tfrom"];
    $cl_finish = $time_rec["tto"];
    $cl_duration = $time_rec["tdur"];

    // This is needed to get the date in user date format (to display on the form).
    $o_date = new DateAndTime(DB_DATEFORMAT, $rec_date);
    $cl_pr_date	= $o_date->toString($i18n->getDateFormat());
    unset($o_date);
    
    $cl_note = $time_rec["al_comment"];
    $cl_billable = $time_rec["al_billable"];

    // Add an info message to the form if we are editing an uncompleted record.
    if ($cl_start == $cl_finish && $cl_duration == "0:00") {
      $cl_finish = "";
      $cl_duration = "";
      $messages->add("no_finished_rec",$i18n->getKey("form.mytime.no_finished_rec"));
    }
  }
  
  // Initialize elements of 'mytimeForm'.
  
  // Start with projects assigned to user.
  import('ProjectHelper');
  $project_list = ProjectHelper::getAssignedProjects($user->getUserId());
  
  $form = new Form('mytimeForm');
  $form->addInput(array("type" => "combobox",
    "onchange" => "fillActivityDropdown(this.value);",
    "name" => "project",
    "style" => "width: 250",
    "value" => @$cl_project,
    "data" => @$project_list,
    "datakeys" => array("p_id", "p_name"),
    "empty" => array("" => $i18n->getKey('controls.select.project'))
  ));
  
  // Now fill activities combo.
  import('ActivityHelper');
  $activity_list = ActivityHelper::getActivities($user->getUserId());
  
  $form->addInput(array("type" => "combobox",
    "name" => "activity",
    "style" => "width: 250",
    "value" => @$cl_activity,
    "data" => @$activity_list,
    "datakeys" => array("a_id", "a_name"),
    "empty" => array("" => $i18n->getKey('controls.select.activity'))
  ));
  
  // Add other controls.
  $form->addInput(array("type" => "text", "name" => "start", "value" => @$cl_start, "onchange" => "formDisable('start');"));
  $form->addInput(array("type" => "text", "name" => "finish", "value" => @$cl_finish, "onchange" => "formDisable('finish');"));
  $form->addInput(array("type" => "text", "name" => "duration", "value" => @$cl_duration, "onchange" => "formDisable('duration');"));
  $form->addInput(array("type" => "datetext", "maxlength" => "20", "name" => "pr_date", "value" => @$cl_pr_date));
  $form->addInput(array("type" => "textarea", "name" => "note", "style" => "width: 250; height: 200;", "value" => $cl_note));
  // Hidden control for a record timestamp.
  $form->addInput(array("type" => "hidden", "name" => "ts", "value" => $cl_ts));
  $form->addInput(array("type" => "checkbox", "name" => "billable", "data" => 1, "value" => $cl_billable));
  $form->addInput(array("type" => "submit", "name" => "btsave", "value" => $i18n->getKey('button.save')));
  $form->addInput(array("type" => "submit", "name" => "btsaveasnew", "value" => $i18n->getKey('button.asnew')));
  $form->addInput(array("type" => "submit", "name" => "btdelete", "value" => $i18n->getKey('button.delete')));
  
  if ($request->getMethod() == "POST") {
    
    // Check input data.
    if (!$cl_project) {
      $errors->add("project", $i18n->getKey("errors.wr_project"));
    }
    if (!$cl_activity) {
      $errors->add("activity", $i18n->getKey("errors.wr_activity"));
    }
    if (!$cl_duration) {
      if ($cl_start || $cl_finish) {
        if (!TimeHelper::isValidTime($cl_start)) {
          $errors->add("start", $i18n->getKey("errors.wrong"), $i18n->getKey("form.mytime.start"));
        }
        if ($cl_finish) {
          if (!TimeHelper::isValidTime($cl_finish)) {
            $errors->add("finish", $i18n->getKey("errors.wrong"), $i18n->getKey("form.mytime.finish"));
          }
          if (!TimeHelper::isCorrectTimeInterval($cl_start, $cl_finish)) {
            $errors->add("interval",$i18n->getKey("errors.wr_interval"));
          }
        }
      } else {
        $errors->add("start",$i18n->getKey("errors.empty"),$i18n->getKey("form.mytime.start"));
        $errors->add("finish",$i18n->getKey("errors.empty"),$i18n->getKey("form.mytime.finish"));
        $errors->add("duration",$i18n->getKey("errors.empty"),$i18n->getKey("form.mytime.duration"));
      }
    } else {
      if(strstr($cl_duration, "."))
        $cl_duration = decimal_to_time($cl_duration);
      if (!TimeHelper::isValidDuration($cl_duration)) {
        $errors->add("duration",$i18n->getKey("errors.wrong"),$i18n->getKey("form.mytime.duration"));
      }
    }
    $validator = new Validator($cl_pr_date);
    $validator->validateDate();
    if (!$validator->isValid()) {
      $errors->add("date", $i18n->getKey("errors.wrong"), $i18n->getKey("form.mytime.date"));
    }
    // Finished checking input data.
    
    // Determine lock date.
    $locktime = $user->getLocktime();
    $lockdate = 0;
    if ($locktime < 0 || $locktime == null || $locktime == "") {
      $sc = new SysConfig(new User($user->getOwnerId(), false));
      $locktime = $sc->getValue(SYSC_LOCK_DAYS);
    }
    if ($locktime > 0) {
      $lockdate = new DateAndTime($i18n->getDateTimeFormat(), time());
      $lockdate->decDay($locktime);
    }
  
    // This is a new date for the time record.
    $new_date = new DateAndTime($i18n->getDateFormat(), $cl_pr_date);

    // Save record.
    if ($request->getParameter('btsave')) {
      // We need to:
      // 1) Prohibit saving locked time entries in any form.
      // 2) Prohibit saving completed unlocked entries into locked interval.
      // 3) Prohibit saving uncompleted unlocked entries when another uncompleted entry exists.
      
      // Now, step by step.
      // 1) Prohibit saving locked time entries in any form.
      if($lockdate && $cr_date->before($lockdate)) {
        $errors->add("lock_period", $i18n->getKey("errors.period_lock"));        
      }
      // 2) Prohibit saving completed unlocked entries into locked interval.
      if($lockdate && $new_date->before($lockdate)) {
        $errors->add("lock_period", $i18n->getKey("errors.period_lock"));        
      }
      // 3) Prohibit saving uncompleted unlocked entries when another uncompleted entry exists.
      $uncompleted = ($cl_finish == "" AND $cl_duration == "");
      if ($uncompleted) {
        $not_completed_rec = TimeHelper::findUncompletedRecord($user->getActiveUser());
        if ($not_completed_rec && ($cl_ts <> $not_completed_rec['al_timestamp'])) {
          // We have another not completed record.
          $errors->add("insert", $i18n->getKey("errors.uncompleted_entry")." <a href = 'mytime_edit.php?ts=".$not_completed_rec['al_timestamp']."'>".$i18n->getKey("errors.goto_record")."</a>");
        }
      }

      // Now, an update.
      if ($errors->isEmpty()) {
        if (TimeHelper::update(array(
          'date' => $new_date->toString(DB_DATEFORMAT),
          'ts' => $cl_ts,
          'user_id' => $user->getActiveUser(),
          'project' => $cl_project,
          'activity' => $cl_activity,
          'start' => $cl_start,
          'finish' => $cl_finish,
          'duration' => $cl_duration,
          'note' => $cl_note,
          'billable' => $cl_billable))) {
            header("Location: mytime.php");
            exit();
          }
      }
    }
      
    // Save as new record.
    if ($request->getParameter('btsaveasnew')) {
      // We need to:
      // 1) Prohibit saving into locked interval.
      // 2) Prohibit saving uncompleted unlocked entries when another uncompleted entry exists.
      
      // Now, step by step.
      // 1) Prohibit saving into locked interval.
      if($lockdate && $new_date->before($lockdate)) {
        $errors->add("lock_period", $i18n->getKey("errors.period_lock"));        
      }
      // 2) Prohibit saving uncompleted unlocked entries when another uncompleted entry exists.
      $uncompleted = ($cl_finish == "" AND $cl_duration == "");
      if ($uncompleted) {
        $not_completed_rec = TimeHelper::findUncompletedRecord($user->getActiveUser());
        if ($not_completed_rec) {
          // We have another not completed record.
          $errors->add("insert", $i18n->getKey("errors.uncompleted_entry")." <a href = 'mytime_edit.php?ts=".$not_completed_rec['al_timestamp']."'>".$i18n->getKey("errors.goto_record")."</a>");
        }
      }

      // Now, a new insert.
      if ($errors->isEmpty()) {
        if (TimeHelper::insert(array(
          'date' => $new_date->toString(DB_DATEFORMAT),
          'user_id' => $user->getActiveUser(),
          'project' => $cl_project,
          'activity' => $cl_activity,
          'start' => $cl_start,
          'finish' => $cl_finish,
          'duration' => $cl_duration,
          'note' => $cl_note,
          'billable' => $cl_billable))) {
            header("Location: mytime.php?date=".date("m/d/Y", strtotime($cl_pr_date))); // This is not working.
            exit();
          }
          $errors->add("insert",$i18n->getKey("errors.mt_insert"));
      }
    }
    if ($request->getParameter('btdelete')) {
    	Header("Location: mytime_del.php?ts=$cl_ts");
        exit();
    }
  } // End of if ($request->getMethod() == "POST")
	
  $week_time = TimeHelper::getTimePerWeek($user, $cr_date);
  
  $smarty->assignByRef("errors", $errors);
  $smarty->assignByRef("messages", $messages);
  $smarty->assign("curr_date", $cr_date->toString());
  $smarty->assignByRef("project_list", $project_list);
  $smarty->assignByRef("activity_list", $activity_list);
  $smarty->assign("forms", array($form->getName() => $form->toArray()));
  $smarty->assign("timestring", TimeHelper::parseTimeString($cr_date));
  $smarty->assign("userdet_string", UserHelper::getUserDetailsString($user,$GLOBALS["I18N"]));
  $smarty->assign("week_time", $week_time);
  $smarty->assign("onload", "onLoad = \"document.mytimeForm.project.focus();fillActivityDropdown(document.mytimeForm.project.value);\"");
  $smarty->assign("title_page", $i18n->getKey("form.mytime.edit_title"));
  $smarty->assign("content_page_name", "mytime_edit.tpl");
  $smarty->display(INDEX_TEMPLATE);
?>